from orjson import dumps as odumps, loads as oloads

from .aiofile import aread_file, asave_file
from .file import read_file, save_file
from .string import camel_to_snake, snake_to_camel
from .typehint import PathOrStr


# Dict

def dict_key_camel_to_snake(data: dict[str]):
    return {camel_to_snake(k): v for k, v in data.items()}


def dict_key_snake_to_camel(data: dict[str]):
    return {snake_to_camel(k): v for k, v in data.items()}


# Json operate

async def aread_json(path: PathOrStr):
    """Async read json file with orjson."""

    return oloads(await aread_file(path))


async def asave_json(path: PathOrStr, data: dict | list):
    """Async save json file with orjson."""

    return await asave_file(path, odumps(data))


def read_json(path: PathOrStr):
    """Read json file with orjson."""

    return oloads(read_file(path))


def save_json(path: PathOrStr, data: dict | list):
    """Save json file with orjson."""

    return save_file(path, odumps(data))



import time

from functools import wraps
from inspect import iscoroutinefunction
from typing import Callable


def show_cost_time(view_func: Callable):
    """Run the function and show cost time.

    Supports async and sync function.
    """

    if iscoroutinefunction(view_func):
        @wraps(view_func)
        async def wrapped_view(*args, **kwargs):
            se = time.time()
            result = await view_func(*args, **kwargs)
            print(f'Function {view_func.__name__} cost time is {time.time() - se} s')
            return result
        return wrapped_view

    @wraps(view_func)
    def wrapped_view(*args, **kwargs):
        se = time.time()
        result = view_func(*args, **kwargs)
        print(f'Function {view_func.__name__} cost time is {time.time() - se} s')
        return result
    return wrapped_view


def try_and_get_bool(view_func):
    """Run the function use try/catch.

    Returns False if there was an error. Otherwise return True.

    Supports async and sync function.
    """

    # Return async wrapped_view if view_func is coro
    if iscoroutinefunction(view_func):
        @wraps(view_func)
        async def wrapped_view(*args, **kwargs):
            try:
                await view_func(*args, **kwargs)
                return True
            except:
                return False
        return wrapped_view

    @wraps(view_func)
    def wrapped_view(*args, **kwargs):
        try:
            view_func(*args, **kwargs)
            return True
        except:
            return False
    return wrapped_view


def try_and_get_data(view_func):
    """Run the function use try/catch.

    Returns None if there was an error. Otherwise return the function result.

    Supports async and sync function.
    """

    # Return async wrapped_view if view_func is coro
    if iscoroutinefunction(view_func):
        @wraps(view_func)
        async def wrapped_view(*args, **kwargs):
            try:
                return await view_func(*args, **kwargs)
            except:
                pass
        return wrapped_view

    @wraps(view_func)
    def wrapped_view(*args, **kwargs):
        try:
            return view_func(*args, **kwargs)
        except:
            pass
    return wrapped_view



import random
import re
import string

from .check import isbytes, isstr


_RANDOM_LETTERS = string.ascii_letters + string.digits


# Case

def camel_to_snake(camel_str: str):
    return re.sub(r'(?<!^)(?=[A-Z])', '_', camel_str).lower()


def snake_to_camel(snake_str: str):
    components = snake_str.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])


def random_str(min_l: int = 8, max_l: int = 8):
    return ''.join(random.choice(_RANDOM_LETTERS) for i in range(random.randint(min_l, max_l)))


def s2b(data: bytes | str, encoding: str = 'utf-8'):
    """Convert string to bytes."""

    if isstr(data):
        return data.encode(encoding)
    if not isbytes(data):
        raise ValueError('Data is not string or bytes!')
    return data


def b2s(data: bytes | str, encoding: str = 'utf-8'):
    """Convert bytes to string."""

    if isbytes(data):
        return data.decode(encoding)
    if not isstr(data):
        raise ValueError('Data is not bytes or string!')
    return data


# Text

def search_text(
    pattern: re.Pattern,
    text: str,
    group_index: int = 0,
    **kwargs
):
    """Search text by passern and return result."""

    result = re.search(pattern, text, **kwargs)
    return result[group_index] if result else None



from socket import gethostbyname
from typing import Optional
from urllib.parse import urlparse

from .check import isdomain
from .decorators import try_and_get_data


# Network utils

@try_and_get_data
def get_domain_ip(domain: str) -> Optional[str]:
    return gethostbyname(domain)


def get_host(url: str):
    """Get the host of the input url."""

    if host := urlparse(url).hostname:
        return host

    return (isdomain(url) and url) or None


def domains_is_ip(domains: list[str], ip: str):
    return all([get_domain_ip(domain) == ip for domain in domains])



import io
import magic
import os
import shutil

from typing import Callable

from .decorators import try_and_get_bool, try_and_get_data
from .typehint import PathOrStr


# File

def clear_dir(path: PathOrStr):
    """Clear dir (Remove and create)."""

    rmdir(path)
    mkdirs(path)


@try_and_get_bool
def del_file(path: PathOrStr):
    """Del file."""

    os.remove(path)


def get_file_mime(file: bytes | io.BytesIO | io.FileIO):
    """Get file mime."""

    is_file = getattr(file, 'read', None) != None
    data = file.read(2048) if is_file else file[:2048]
    file_mime = magic.from_buffer(data, mime=True)

    if is_file:
        file.seek(0)

    return file_mime.split('/')


@try_and_get_data
def get_file_size(path: PathOrStr):
    return os.stat(path).st_size


@try_and_get_bool
def mkdir(path: PathOrStr):
    """Create dir."""

    os.mkdir(path)


@try_and_get_bool
def mkdirs(path: PathOrStr):
    """Create dir (use makedirs)."""

    os.makedirs(path, exist_ok=True)


@try_and_get_bool
def move_file(path: PathOrStr, target_path: PathOrStr):
    """Move file or dir."""

    shutil.move(path, target_path)


@try_and_get_data
def read_file(path: PathOrStr):
    """Read file."""

    with open(path, 'rb') as f:
        return f.read()


@try_and_get_bool
def rename(path: PathOrStr, name: PathOrStr):
    """Rename file or dir."""

    os.rename(path, name)


@try_and_get_bool
def rmdir(path: PathOrStr):
    """Remove dir."""

    shutil.rmtree(path)


@try_and_get_data
def save_file(
    path: PathOrStr,
    file: bytes | io.BytesIO | io.FileIO | str,
    replace: bool = True
):
    """Save file."""

    mode = 'w' if isinstance(file, str) else 'wb'

    if os.path.exists(path) and not replace:
        raise FileExistsError()
    if getattr(file, 'read', None):
        file = file.read()
    with open(path, mode) as f:
        return f.write(file)


def save_file_as_bytesio(
    save_fnc: Callable,
    get_bytes: bool = False,
    **kwargs
):
    """Save file to io.BytesIO."""

    with io.BytesIO() as output:
        save_fnc(output, **kwargs)
        file_bytes = output.getvalue()

    if get_bytes:
        return file_bytes

    return io.BytesIO(file_bytes)



import aioshutil
import io

from aiofiles import open as aopen, os as aos

from .decorators import try_and_get_bool, try_and_get_data
from .typehint import PathOrStr


# Async File

async def aclear_dir(path: PathOrStr):
    """Async clear dir (Remove and create)."""

    await armdir(path)
    await amkdirs(path)


@try_and_get_bool
async def adel_file(path: PathOrStr):
    """Async del file."""

    await aos.remove(path)


@try_and_get_data
async def aget_file_size(path: PathOrStr):
    """Async get file size."""

    return (await aos.stat(path)).st_size


@try_and_get_bool
async def amkdir(path: PathOrStr):
    """Async create dir."""

    await aos.mkdir(path)


@try_and_get_bool
async def amkdirs(path: PathOrStr):
    """Async create dir (use makedirs)."""

    await aos.makedirs(path, exist_ok=True)


@try_and_get_bool
async def amove_file(path: PathOrStr, target_path: PathOrStr):
    """Move file or dir."""

    await aioshutil.move(path, target_path)


@try_and_get_data
async def aread_file(path: PathOrStr, **kwargs):
    """Async read file."""

    async with aopen(path, 'rb', **kwargs) as f:
        return await f.read()


@try_and_get_bool
async def arename(path: PathOrStr, name: PathOrStr):
    """Async rename file or dir."""

    await aos.rename(path, name)


@try_and_get_bool
async def armdir(path: PathOrStr):
    """Async Remove dir."""

    await aioshutil.rmtree(path)


@try_and_get_data
async def asave_file(
    path: PathOrStr,
    file: bytes | io.BytesIO | io.FileIO | str,
    replace: bool = True,
    **kwargs
):
    """Async save file."""

    mode = 'w' if isinstance(file, str) else 'wb'

    if await aos.path.exists(path) and not replace:
        raise FileExistsError()
    if getattr(file, 'read', None):
        file = file.read()
    async with aopen(path, mode, **kwargs) as f:
        return await f.write(file)



from binascii import a2b_hex
from Cryptodome.Cipher.AES import block_size, MODE_CBC, MODE_CFB, MODE_CTR, MODE_ECB, new
from typing import Optional

from .json import odumps, oloads


class AesCrypt:
    """Padding mode is PKCS7 Padding."""

    def __init__(
        self,
        key: bytes | str,
        iv: Optional[bytes | str] = None,
        mode=MODE_CBC,
        counter=None
    ):
        if isinstance(key, str):
            key = key.encode()

        if isinstance(iv, str):
            iv = iv.encode()

        if mode == MODE_ECB:
            self._get_aes = lambda: new(key, mode)
        elif mode == MODE_CTR:
            self._get_aes = lambda: new(key, mode, counter=counter)
        else:
            self._get_aes = lambda: new(key, mode, iv)

        if mode == MODE_CFB:
            self._pad = self._rstrip = lambda x: x
        else:
            def _pad(data: bytes):
                n = block_size - (len(data) % block_size)
                return data + (chr(n).encode() * n)

            self._pad = _pad
            self._unpad = lambda x: x[:len(x)-x[-1]]

    @staticmethod
    def _to_bytes(data: bytes | dict | list | str) -> bytes:
        if isinstance(data, bytes):
            return data

        if isinstance(data, (dict, list)):
            return odumps(data)

        return data.encode('utf-8')

    def decrypt(self, ciphertext: bytes | str) -> dict | list | str:
        if isinstance(ciphertext, str):
            ciphertext = ciphertext.encode('utf-8')

        ciphertext = a2b_hex(ciphertext)
        text: bytes = self._unpad(self._get_aes().decrypt(ciphertext))

        try:
            return oloads(text)
        except:
            return text.decode()

    def encrypt(self, data: bytes | dict | list | str, return_bytes: bool = False):
        encrypted_data = self._get_aes().encrypt(self._pad(self._to_bytes(data)))
        return encrypted_data if return_bytes else encrypted_data.hex()



from uuid import uuid1

from .aiofile import aread_file, asave_file
from .file import read_file, save_file
from .typehint import PathOrStr


# UUID

async def aget_uuid(save_path: PathOrStr = './uuid.uuid'):
    if now_uuid := await aread_file(save_path):
        return now_uuid.decode()

    now_uuid = str(uuid1())
    await asave_file(save_path, now_uuid)
    return now_uuid


def get_uuid(save_path: PathOrStr = './uuid.uuid'):
    if now_uuid := read_file(save_path):
        return now_uuid.decode()

    now_uuid = str(uuid1())
    save_file(save_path, now_uuid)
    return now_uuid



import os
import re

from .typehint import PathOrStr


ALLOWED_EMAILS = [
    'gmail.com',
    'yahoo.com',
    'hotmail.com',
    'aol.com',
    'hotmail.co.uk',
    'hotmail.fr',
    'msn.com',
    'wanadoo.fr',
    'live.com',
    'hotmail.it',
    'qq.com'
]

domain_pattern = re.compile(
    r'^(?:[a-zA-Z0-9]'  # First character of the domain
    r'(?:[a-zA-Z0-9-_]{0,61}[A-Za-z0-9])?\.)'  # Sub domain + hostname
    r'+[A-Za-z0-9][A-Za-z0-9-_]{0,61}'  # First 61 characters of the gTLD
    r'[A-Za-z]$'  # Last character of the gTLD
)


# Check

def isbytes(*args):
    """Determine whether it is bytes."""

    return all([isinstance(arg, bytes) for arg in args])


def isdomain(domain: str):
    """Check domain."""

    return domain_pattern.match(domain) is not None


def isemail(email: str):
    """Check email format and ping the domain."""

    if re.match(r'.*[+\-*/\\;&|\s​].*', email):
        return False

    domain = email.split('@')[-1].lower()
    return domain in ALLOWED_EMAILS or isdomain(domain)


def isdict(*args):
    """Determine whether it is dict."""

    return all([isinstance(arg, dict) for arg in args])


def isdir(*args: PathOrStr):
    """Determine whether path is dir."""

    return all([os.path.isdir(arg) for arg in args])


def isfile(*args: PathOrStr):
    """Determine whether path is file."""

    return all([os.path.isfile(arg) for arg in args])


def isint(*args):
    """Determine whether it is int."""

    return all([isinstance(arg, int) for arg in args])


def islist(*args):
    """Determine whether it is list."""

    return all([isinstance(arg, list) for arg in args])


def isstr(*args):
    """Determine whether it is str."""

    return all([isinstance(arg, str) for arg in args])
